Chromium Code Reviews
chromiumcodereview-hr@appspot.gserviceaccount.com (chromiumcodereview-hr) | Please choose your nickname with Settings | Help | Chromium Project | Gerrit Changes | Sign out
(3157)

Unified Diff: base/debug/task_annotator.cc

Issue 1044413002: Record async "task backtraces" (Closed) Base URL: https://chromium.googlesource.com/chromium/src.git@master
Patch Set: More wording fixes. Created 3 years, 10 months ago
Use n/p to move between diff chunks; N/P to move between comments. Draft comments are only viewable by you.
Jump to:
View side-by-side diff with in-line comments
Download patch
« no previous file with comments | « base/BUILD.gn ('k') | base/message_loop/incoming_task_queue.cc » ('j') | no next file with comments »
Expand Comments ('e') | Collapse Comments ('c') | Show Comments Hide Comments ('s')
Index: base/debug/task_annotator.cc
diff --git a/base/debug/task_annotator.cc b/base/debug/task_annotator.cc
index 437d69a7f8be025b17dabe29adb12c7dedc63816..46969f28ca8cb705ed2d7440bbb21a026148a0b8 100644
--- a/base/debug/task_annotator.cc
+++ b/base/debug/task_annotator.cc
@@ -4,6 +4,8 @@
#include "base/debug/task_annotator.h"
+#include <array>
+
#include "base/debug/activity_tracker.h"
#include "base/debug/alias.h"
#include "base/pending_task.h"
@@ -41,13 +43,18 @@ void TaskAnnotator::RunTask(const char* queue_function,
TRACE_ID_MANGLE(GetTaskTraceID(*pending_task)), TRACE_EVENT_FLAG_FLOW_IN,
"queue_duration", queue_duration.InMilliseconds());
- // Before running the task, store the program counter where it was posted
- // and deliberately alias it to ensure it is on the stack if the task
- // crashes. Be careful not to assume that the variable itself will have the
- // expected value when displayed by the optimizer in an optimized build.
- // Look at a memory dump of the stack.
- const void* program_counter = pending_task->posted_from.program_counter();
- debug::Alias(&program_counter);
+ // Before running the task, store the task backtrace with the chain of
+ // PostTasks that resulted in this call and deliberately alias it to ensure
+ // it is on the stack if the task crashes. Be careful not to assume that the
+ // variable itself will have the expected value when displayed by the
+ // optimizer in an optimized build. Look at a memory dump of the stack.
+ static constexpr int kStackTaskTraceSnapshotSize =
+ std::tuple_size<decltype(pending_task->task_backtrace)>::value + 1;
+ std::array<const void*, kStackTaskTraceSnapshotSize> task_backtrace;
+ task_backtrace[0] = pending_task->posted_from.program_counter();
+ std::copy(pending_task->task_backtrace.begin(),
+ pending_task->task_backtrace.end(), task_backtrace.begin() + 1);
+ debug::Alias(&task_backtrace);
std::move(pending_task->task).Run();
« no previous file with comments | « base/BUILD.gn ('k') | base/message_loop/incoming_task_queue.cc » ('j') | no next file with comments »

Powered by Google App Engine
This is Rietveld 408576698